//////////////////////////////////////////////////////////////////////         
// rotatingHelix3.cpp
//
// This program, based on helix.cpp, animates a helix by rotating
// it around its axis using a timer function called by glutTimerFunc().
//
// Interaction:
// Press space to toggle between animation on and off.
// Press the up/down arrow keys to speed up/slow down animation.
//
//Sumanta Guha.
////////////////////////////////////////////////////////////////////// 

#define _USE_MATH_DEFINES 

#include <cstdlib>
#include <cmath>
#include <iostream>

#include <GL/glew.h>
#include <GL/freeglut.h> 

// Globals.
static int isAnimate = 0; // Animated?
static int animationPeriod = 50; // Time interval between frames.
static float angle = 0.0; // Angle of rotation.

// Drawing routine.
void drawScene(void)
{
    float R = 20.0; // Radius of helix.

    float t; // Angle parameter along helix.

    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(0.0, 0.0, 0.0);
    glPushMatrix();

    // The Trick: to align the axis of the helix along the y-axis prior to
    // rotation and then return it to its original location.
    glTranslatef(0.0, 0.0, -60.0);
    glRotatef(angle, 0.0, 1.0, 0.0);
    glTranslatef(0.0, 0.0, 60.0);

    glBegin(GL_LINE_STRIP);
    for (t = -10 * M_PI; t <= 10 * M_PI; t += M_PI / 20.0)
        glVertex3f(R * cos(t), t, R * sin(t) - 60.0);
    glEnd();

    glPopMatrix();
    glutSwapBuffers();
}

// Initialization routine.
void setup(void)
{
    glClearColor(1.0, 1.0, 1.0, 0.0);
}

// OpenGL window reshape routine.
void resize(int w, int h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-5.0, 5.0, -5.0, 5.0, 5.0, 100.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

// Routine to increase the rotation angle.
void increaseAngle(void)
{
    angle += 5.0;
    if (angle > 360.0)
        angle -= 360.0;
}

// Routine to animate with a recursive call made after animationPeriod msecs.
void animate(int value)
{
    if (isAnimate)
    {
        increaseAngle();

        glutPostRedisplay();
        glutTimerFunc(animationPeriod, animate, 1);
    }
}

// Keyboard input processing routine.
void keyInput(unsigned char key, int x, int y)
{
    switch (key)
    {
    case 27:
        exit(0);
        break;
    case ' ':
        if (isAnimate) isAnimate = 0;
        else
        {
            isAnimate = 1;
            animate(1);
        }
        break;
    default:
        break;
    }
}

// Callback routine for non-ASCII key entry.
void specialKeyInput(int key, int x, int y)
{
    if (key == GLUT_KEY_DOWN) animationPeriod += 5;
    if (key == GLUT_KEY_UP)
        if (animationPeriod > 5) animationPeriod -= 5;
    glutPostRedisplay();
}

// Routine to output interaction instructions to the C++ window.
void printInteraction(void)
{
    std::cout << "Interaction:" << std::endl;
    std::cout << "Press space to toggle between animation on and off."
              << std::endl
              << "Press the up/down arrow keys to speed up/slow down"
              << " animation." << std::endl;
}

// Main routine.
int main(int argc, char **argv)
{
    printInteraction();
    glutInit(&argc, argv);

    glutInitContextVersion(3, 1);
    glutInitContextProfile(GLUT_COMPATIBILITY_PROFILE);

    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowSize(500, 500);
    glutInitWindowPosition(100, 100);
    glutCreateWindow("rotatingHelix3.cpp");
    glutDisplayFunc(drawScene);
    glutReshapeFunc(resize);
    glutKeyboardFunc(keyInput);
    glutSpecialFunc(specialKeyInput);

    glewExperimental = GL_TRUE;
    glewInit();

    setup();

    glutMainLoop();
}
